/******** header files ********/
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

/****** local header files ******/
#include "gfx_functions.h"

/******* definitions ********/
#define DEFAULT_IMAGE "penguinball.png"
/******** function declarations  ********/
int process_arguments(int argc, char *argv[]);
void initialize_screen(void);
void handle_events(void);

/****************** global variables ******************/
int mousebutton_state, mousex, mousey;
char *imagefilename = NULL;
SDL_Surface *image;
int videoflags = 0;

/******** function definitions ***********/
int main(int argc, char *argv[]){
	int readpoint[8], writepoint[8], xi[4], yi[4];
	int n;

	atexit(&gfx_free_memory);


	if(argc > 1){
		if(process_arguments(argc, argv)) exit(1);
	}
	if(imagefilename == NULL){
		image = gfx_load_png(DEFAULT_IMAGE);
	}else{
		image = gfx_load_png(imagefilename);
	}
	if(image == (SDL_Surface *)NULL) exit(1);
	else gfx_add_freepointer(image);

	initialize_screen();

	writepoint[0] = 0;
	writepoint[1] = x_resolution - 1;
	writepoint[2] = x_resolution - 1;
	writepoint[3] = 0;
	writepoint[4] = 0;
	writepoint[5] = 0;
	writepoint[6] = y_resolution - 1;
	writepoint[7] = y_resolution - 1;
	readpoint[0] = 0;
	readpoint[1] = image->w - 1;
	readpoint[2] = image->w - 1;
	readpoint[3] = 0;
	readpoint[4] = 0;
	readpoint[5] = 0;
	readpoint[6] = image->h - 1;
	readpoint[7] = image->h - 1;

	do{
	      for(n = 0; n < 4; n++){
			if(!(rand() % 500)) xi[n] *= -1;
			if(!(rand() % 500)) yi[n] *= -1;
			if(readpoint[n] + xi[n] < 0) xi[n] = 10;
			if(readpoint[n] + xi[n] >= image->w) xi[n] = -10;
			if(readpoint[n + 4] + yi[n] < 0) yi[n] = 10;
			if(readpoint[n + 4] + yi[n] >= image->h) yi[n] = -10;
			readpoint[n] += xi[n];
			readpoint[n + 4] += yi[n];
		}
		gfx_draw_texture(screen_image, writepoint, readpoint, 4, image);
		SDL_UpdateRect(screen_image, 0, 0, 0, 0);
		handle_events();
	}while(mousebutton_state != 1);

	return 0;
}

int process_arguments(int argc, char *argv[]){
	int n, returnval = 0;

	for(n = 1; n < argc; n++){
		if(!(strcmp(argv[n], "-i") && strcmp(argv[n], "--image"))){
			if(argc - 1 > n){
				n++;
				imagefilename = argv[n];
			}else{
				fprintf(stderr, "%s argument must be followed with an image name.\n", argv[n]);
				returnval |= 1;
			}
		}else if(!(strcmp(argv[n], "-f") && strcmp(argv[n], "--fullscreen"))){
			videoflags |= SDL_FULLSCREEN;
		}else if(!(strcmp(argv[n], "-h") && strcmp(argv[n], "--help"))){
			printf("Syntax:\twarp <arguments>\n");
			printf("Valid arguments are:\n");
			printf("\n-i filename\t--image filename\nspecify the image file to use for the demo.  The image must be in a .png format.\n");
			printf("\n-f\t--fullscreen\nrun in full screen mode if possible.\n");
			printf("\n-h\t--help\nShow this help and exit.\n");
			printf("\nIf no arguments are specified, the program is run in windowed mode and will attempt do load the default image, \"%s\", from the present working directory.\n", DEFAULT_IMAGE);
			returnval |= 2;
		}else{
			fprintf(stderr, "Unrecognized argument: %s\nuse --help for more info.\n", argv[n]);
			returnval |= 4;
		}
	}
	return returnval;
}

void initialize_screen(void){
	/** initialize the graphics **/
	if(SDL_Init(SDL_INIT_VIDEO) < 0){
		/** can't initialize SDL **/
		fprintf(stderr, "Unable to initialize SDL\n");
		exit(1);
	}

	atexit(SDL_Quit);
	if(gfx_init(640, 480, 32, videoflags)){
		fprintf(stderr, "Unable to initialize graphics\n");
		exit(1);
	}
}

void handle_events(void){
	SDL_Event event;
	while(SDL_PollEvent(&event)){
		switch(event.type){
			case SDL_MOUSEBUTTONUP:
				if(event.button.button == SDL_BUTTON_LEFT) mousebutton_state &= 254; 
				if(event.button.button == SDL_BUTTON_RIGHT) mousebutton_state &= 253; 
				if(event.button.button == SDL_BUTTON_MIDDLE) mousebutton_state &= 251; 
				break;  
			case SDL_MOUSEBUTTONDOWN:
				if(event.button.button == SDL_BUTTON_LEFT) mousebutton_state |= 1;
				if(event.button.button == SDL_BUTTON_RIGHT) mousebutton_state |= 2;
				if(event.button.button == SDL_BUTTON_MIDDLE) mousebutton_state |= 4;
				break;  
			case SDL_MOUSEMOTION:
				mousex = event.motion.x;
				mousey = event.motion.y;
				break;  
			default:
				break;  
		}
	}       
}
